@@ -22,9 +22,9 @@
  */
 
 /**
- * SECTION:clutter-path
- * @short_description: An object describing a path with straight lines
- * and bezier curves.
+ * ClutterPath:
+ * 
+ * An object describing a path with straight lines and bezier curves.
  *
  * A #ClutterPath contains a description of a path consisting of
  * straight lines and bezier curves.
@@ -42,7 +42,7 @@
  *  - %CLUTTER_PATH_CURVE_TO, creates a bezier curve. The end of the
  *  last node is used as the first control point and the three
  *  subsequent coordinates given in the node as used as the other three.
- *  -%CLUTTER_PATH_CLOSE, creates a straight line from the last node to
+ *  - %CLUTTER_PATH_CLOSE, creates a straight line from the last node to
  *  the last %CLUTTER_PATH_MOVE_TO node. This can be used to close a
  *  path so that it will appear as a loop when animated.
  *
@@ -53,11 +53,9 @@
  * direct screen positions.
  *
  * You can build a path using the node adding functions such as
- * clutter_path_add_line_to(). Alternatively the path can be described
+ * [method@Path.add_line_to]. Alternatively the path can be described
  * in a string using a subset of the SVG path syntax. See
- * clutter_path_add_string() for details.
- *
- * #ClutterPath is available since Clutter 1.0
+ * [method@Path.add_string] for details.
  */
 
 #include "clutter-build-config.h"
@@ -213,19 +211,26 @@ static void
 clutter_value_transform_path_string (const GValue *src,
                                      GValue       *dest)
 {
-  gchar *string = clutter_path_get_description (src->data[0].v_pointer);
+  if (src->data[0].v_pointer != NULL)
+    {
+      gchar *string = clutter_path_get_description (src->data[0].v_pointer);
 
-  g_value_take_string (dest, string);
+      g_value_take_string (dest, string);
+    }
 }
 
 static void
 clutter_value_transform_string_path (const GValue *src,
                                      GValue       *dest)
 {
-  ClutterPath *new_path;
+  const char *str;
 
-  new_path = clutter_path_new_with_description (g_value_get_string (src));
-  g_value_take_object (dest, new_path);
+  str = g_value_get_string (src);
+  if (str != NULL)
+    {
+      ClutterPath *new_path = clutter_path_new_with_description (str);
+      g_value_take_object (dest, new_path);
+    }
 }
 
 static void
@@ -244,8 +249,6 @@ clutter_path_finalize (GObject *object)
  * Creates a new #ClutterPath instance with no nodes.
  *
  * Return value: the newly created #ClutterPath
- *
- * Since: 1.0
  */
 ClutterPath *
 clutter_path_new (void)
@@ -260,12 +263,10 @@ clutter_path_new (void)
  * @desc: a string describing the path
  *
  * Creates a new #ClutterPath instance with the nodes described in
- * @desc. See clutter_path_add_string() for details of the format of
+ * @desc. See [method@Path.add_string] for details of the format of
  * the string.
  *
  * Return value: the newly created #ClutterPath
- *
- * Since: 1.0
  */
 ClutterPath *
 clutter_path_new_with_description (const gchar *desc)
@@ -280,8 +281,6 @@ clutter_path_new_with_description (const gchar *desc)
  * @path: a #ClutterPath
  *
  * Removes all nodes from the path.
- *
- * Since: 1.0
  */
 void
 clutter_path_clear (ClutterPath *path)
@@ -351,8 +350,6 @@ clutter_path_add_node_helper (ClutterPath         *path,
  * Adds a %CLUTTER_PATH_MOVE_TO type node to the path. This is usually
  * used as the first node in a path. It can also be used in the middle
  * of the path to cause the actor to jump to the new coordinate.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_move_to (ClutterPath *path,
@@ -370,10 +367,8 @@ clutter_path_add_move_to (ClutterPath *path,
  * @x: the x coordinate
  * @y: the y coordinate
  *
- * Same as clutter_path_add_move_to() except the coordinates are
+ * Same as [method@Path.add_move_to] except the coordinates are
  * relative to the previous node.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_rel_move_to (ClutterPath *path,
@@ -393,8 +388,6 @@ clutter_path_add_rel_move_to (ClutterPath *path,
  *
  * Adds a %CLUTTER_PATH_LINE_TO type node to the path. This causes the
  * actor to move to the new coordinates in a straight line.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_line_to (ClutterPath *path,
@@ -412,10 +405,8 @@ clutter_path_add_line_to (ClutterPath *path,
  * @x: the x coordinate
  * @y: the y coordinate
  *
- * Same as clutter_path_add_line_to() except the coordinates are
+ * Same as [method@Path.add_line_to] except the coordinates are
  * relative to the previous node.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_rel_line_to (ClutterPath *path,
@@ -440,8 +431,6 @@ clutter_path_add_rel_line_to (ClutterPath *path,
  * Adds a %CLUTTER_PATH_CURVE_TO type node to the path. This causes
  * the actor to follow a bezier from the last node to (@x_3, @y_3) using
  * (@x_1, @y_1) and (@x_2,@y_2) as control points.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_curve_to (ClutterPath *path,
@@ -470,10 +459,8 @@ clutter_path_add_curve_to (ClutterPath *path,
  * @x_3: the x coordinate of the third control point
  * @y_3: the y coordinate of the third control point
  *
- * Same as clutter_path_add_curve_to() except the coordinates are
+ * Same as [method@Path.add_curve_to] except the coordinates are
  * relative to the previous node.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_rel_curve_to (ClutterPath *path,
@@ -499,8 +486,6 @@ clutter_path_add_rel_curve_to (ClutterPath *path,
  * Adds a %CLUTTER_PATH_CLOSE type node to the path. This creates a
  * straight line from the last node to the last %CLUTTER_PATH_MOVE_TO
  * type node.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_close (ClutterPath *path)
@@ -698,17 +683,15 @@ clutter_path_add_nodes (ClutterPath *path,
  * For example, to move an actor in a 100 by 100 pixel square centered
  * on the point 300,300 you could use the following path:
  *
- * |[
+ * ```
  *   M 250,350 l 0 -100 L 350,250 l 0 100 z
- * ]|
+ * ```
  *
  * If the path description isn't valid %FALSE will be returned and no
  * nodes will be added.
  *
  * Return value: %TRUE is the path description was valid or %FALSE
  * otherwise.
- *
- * Since: 1.0
  */
 gboolean
 clutter_path_add_string (ClutterPath *path,
@@ -735,8 +718,6 @@ clutter_path_add_string (ClutterPath *path,
  * @node: a #ClutterPathNode
  *
  * Adds @node to the end of the path.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_node (ClutterPath           *path,
@@ -760,8 +741,6 @@ clutter_path_add_node (ClutterPath           *path,
  * @cpath: a Cairo path
  *
  * Add the nodes of the Cairo path to the end of @path.
- *
- * Since: 1.0
  */
 void
 clutter_path_add_cairo_path (ClutterPath        *path,
@@ -856,8 +835,6 @@ clutter_path_add_node_to_cairo_path (const ClutterPathNode *node,
  * @cr: a Cairo context
  *
  * Add the nodes of the ClutterPath to the path in the Cairo context.
- *
- * Since: 1.0
  */
 void
 clutter_path_to_cairo_path (ClutterPath *path,
@@ -876,8 +853,6 @@ clutter_path_to_cairo_path (ClutterPath *path,
  * Retrieves the number of nodes in the path.
  *
  * Return value: the number of nodes.
- *
- * Since: 1.0
  */
 guint
 clutter_path_get_n_nodes (ClutterPath *path)
@@ -898,8 +873,6 @@ clutter_path_get_n_nodes (ClutterPath *path)
  * @node: (out): a location to store a copy of the node
  *
  * Retrieves the node of the path indexed by @index.
- *
- * Since: 1.0
  */
 void
 clutter_path_get_node (ClutterPath     *path,
@@ -924,16 +897,15 @@ clutter_path_get_node (ClutterPath     *path,
  * clutter_path_get_nodes:
  * @path: a #ClutterPath
  *
- * Returns a #GSList of #ClutterPathNode<!-- -->s. The list should be
- * freed with g_slist_free(). The nodes are owned by the path and
- * should not be freed. Altering the path may cause the nodes in the
- * list to become invalid so you should copy them if you want to keep
- * the list.
+ * Returns a #GSList of [struct@PathNode]s.
+ * 
+ * The list should be freed with g_slist_free(). The nodes are owned 
+ * by the path and should not be freed. Altering the path may cause
+ * the nodes in the list to become invalid so you should copy them
+ * if you want to keep the list.
  *
  * Return value: (transfer container) (element-type Clutter.PathNode): a
  *   list of nodes in the path.
- *
- * Since: 1.0
  */
 GSList *
 clutter_path_get_nodes (ClutterPath *path)
@@ -954,8 +926,6 @@ clutter_path_get_nodes (ClutterPath *path)
  * @user_data: user data to pass to the function
  *
  * Calls a function for each node of the path.
- *
- * Since: 1.0
  */
 void
 clutter_path_foreach (ClutterPath         *path,
@@ -979,8 +949,6 @@ clutter_path_foreach (ClutterPath         *path,
  *
  * Inserts @node into the path before the node at the given offset. If
  * @index_ is negative it will append the node to the end of the path.
- *
- * Since: 1.0
  */
 void
 clutter_path_insert_node (ClutterPath           *path,
@@ -1015,8 +983,6 @@ clutter_path_insert_node (ClutterPath           *path,
  * @index_: index of the node to remove
  *
  * Removes the node at the given offset from the path.
- *
- * Since: 1.0
  */
 void
 clutter_path_remove_node (ClutterPath *path,
@@ -1057,8 +1023,6 @@ clutter_path_remove_node (ClutterPath *path,
  * @node: the replacement node
  *
  * Replaces the node at offset @index_ with @node.
- *
- * Since: 1.0
  */
 void
 clutter_path_replace_node (ClutterPath           *path,
@@ -1088,14 +1052,12 @@ clutter_path_replace_node (ClutterPath           *path,
  * @str: a string describing the path
  *
  * Replaces all of the nodes in the path with nodes described by
- * @str. See clutter_path_add_string() for details of the format.
+ * @str. See [method@Path.add_string] for details of the format.
  *
  * If the string is invalid then %FALSE is returned and the path is
  * unaltered.
  *
  * Return value: %TRUE is the path was valid, %FALSE otherwise.
- *
- * Since: 1.0
  */
 gboolean
 clutter_path_set_description (ClutterPath *path,
@@ -1122,11 +1084,9 @@ clutter_path_set_description (ClutterPath *path,
  * @path: a #ClutterPath
  *
  * Returns a newly allocated string describing the path in the same
- * format as used by clutter_path_add_string().
+ * format as used by [method@Path.add_string].
  *
  * Return value: a string description of the path. Free with g_free().
- *
- * Since: 1.0
  */
 gchar *
 clutter_path_get_description (ClutterPath *path)
@@ -1338,8 +1298,6 @@ clutter_path_ensure_node_data (ClutterPath *path)
  * interpolated position is then stored in @position.
  *
  * Return value: index of the node used to calculate the position.
- *
- * Since: 1.0
  */
 guint
 clutter_path_get_position (ClutterPath *path,
@@ -1430,8 +1388,6 @@ clutter_path_get_position (ClutterPath *path,
  * Retrieves an approximation of the total length of the path.
  *
  * Return value: the length of the path.
- *
- * Since: 1.0
  */
 guint
 clutter_path_get_length (ClutterPath *path)
@@ -1465,8 +1421,6 @@ clutter_path_node_full_free (ClutterPathNodeFull *node)
  * Makes an allocated copy of a node.
  *
  * Return value: the copied node.
- *
- * Since: 1.0
  */
 ClutterPathNode *
 clutter_path_node_copy (const ClutterPathNode *node)
@@ -1479,8 +1433,6 @@ clutter_path_node_copy (const ClutterPathNode *node)
  * @node: a #ClutterPathNode
  *
  * Frees the memory of an allocated node.
- *
- * Since: 1.0
  */
 void
 clutter_path_node_free (ClutterPathNode *node)
@@ -1498,8 +1450,6 @@ clutter_path_node_free (ClutterPathNode *node)
  * same coordinates.
  *
  * Return value: %TRUE if the nodes are the same.
- *
- * Since: 1.0
  */
 gboolean
 clutter_path_node_equal (const ClutterPathNode *node_a,
@@ -1541,8 +1491,6 @@ G_DEFINE_BOXED_TYPE (ClutterKnot, clutter_knot,
  * Makes an allocated copy of a knot.
  *
  * Return value: the copied knot.
- *
- * Since: 0.2
  */
 ClutterKnot *
 clutter_knot_copy (const ClutterKnot *knot)
@@ -1558,8 +1506,6 @@ clutter_knot_copy (const ClutterKnot *knot)
  * @knot: a #ClutterKnot
  *
  * Frees the memory of an allocated knot.
- *
- * Since: 0.2
  */
 void
 clutter_knot_free (ClutterKnot *knot)
@@ -1576,8 +1522,6 @@ clutter_knot_free (ClutterKnot *knot)
  * Compares to knot and checks if the point to the same location.
  *
  * Return value: %TRUE if the knots point to the same location.
- *
- * Since: 0.2
  */
 gboolean
 clutter_knot_equal (const ClutterKnot *knot_a,
