@@ -73,6 +73,7 @@ typedef struct _CoglTexturePixmapX11 CoglTexturePixmapX11;
  *
  * Returns: a #GType that can be used with the GLib type system.
  */
+COGL_EXPORT
 GType cogl_texture_pixmap_x11_get_gtype (void);
 
 typedef enum
@@ -87,8 +88,6 @@ typedef enum
  * COGL_TEXTURE_PIXMAP_X11_ERROR:
  *
  * #GError domain for texture-pixmap-x11 errors.
- *
- * Since: 1.10
  */
 #define COGL_TEXTURE_PIXMAP_X11_ERROR (cogl_texture_pixmap_x11_error_quark ())
 
@@ -98,14 +97,13 @@ typedef enum
  *
  * Error codes that can be thrown when performing texture-pixmap-x11
  * operations.
- *
- * Since: 1.10
  */
 typedef enum
 {
   COGL_TEXTURE_PIXMAP_X11_ERROR_X11,
 } CoglTexturePixmapX11Error;
 
+COGL_EXPORT
 uint32_t cogl_texture_pixmap_x11_error_quark (void);
 
 /**
@@ -122,11 +120,8 @@ uint32_t cogl_texture_pixmap_x11_error_quark (void);
  * when it changes.
  *
  * Return value: a new #CoglTexturePixmapX11 instance
- *
- * Since: 1.10
- * Stability: Unstable
  */
-CoglTexturePixmapX11 *
+COGL_EXPORT CoglTexturePixmapX11 *
 cogl_texture_pixmap_x11_new (CoglContext *context,
                              uint32_t pixmap,
                              gboolean automatic_updates,
@@ -161,11 +156,8 @@ cogl_texture_pixmap_x11_new (CoglContext *context,
  * pixmap is not necessary, but may save resources.)
  *
  * Return value: a new #CoglTexturePixmapX11 instance
- *
- * Since: 1.20
- * Stability: Unstable
  */
-CoglTexturePixmapX11 *
+COGL_EXPORT CoglTexturePixmapX11 *
 cogl_texture_pixmap_x11_new_left (CoglContext *context,
                                   uint32_t pixmap,
                                   gboolean automatic_updates,
@@ -181,11 +173,8 @@ cogl_texture_pixmap_x11_new_left (CoglContext *context,
  * created using cogl_texture_pixmap_x11_new_left().
  *
  * Return value: a new #CoglTexturePixmapX11 instance
- *
- * Since: 1.20
- * Stability: Unstable
  */
-CoglTexturePixmapX11 *
+COGL_EXPORT CoglTexturePixmapX11 *
 cogl_texture_pixmap_x11_new_right (CoglTexturePixmapX11 *left_texture);
 
 /**
@@ -199,11 +188,8 @@ cogl_texture_pixmap_x11_new_right (CoglTexturePixmapX11 *left_texture);
  * Forces an update of the given @texture so that it is refreshed with
  * the contents of the pixmap that was given to
  * cogl_texture_pixmap_x11_new().
- *
- * Since: 1.4
- * Stability: Unstable
  */
-void
+COGL_EXPORT void
 cogl_texture_pixmap_x11_update_area (CoglTexturePixmapX11 *texture,
                                      int x,
                                      int y,
@@ -222,11 +208,8 @@ cogl_texture_pixmap_x11_update_area (CoglTexturePixmapX11 *texture,
  *
  * Return value: %TRUE if the texture is using an efficient extension
  *   and %FALSE otherwise
- *
- * Since: 1.4
- * Stability: Unstable
  */
-gboolean
+COGL_EXPORT gboolean
 cogl_texture_pixmap_x11_is_using_tfp_extension (CoglTexturePixmapX11 *texture);
 
 /**
@@ -237,11 +220,8 @@ cogl_texture_pixmap_x11_is_using_tfp_extension (CoglTexturePixmapX11 *texture);
  *
  * Return value: %TRUE if the object is a #CoglTexturePixmapX11, and
  *   %FALSE otherwise
- *
- * Since: 1.4
- * Stability: Unstable
  */
-gboolean
+COGL_EXPORT gboolean
 cogl_is_texture_pixmap_x11 (void *object);
 
 G_END_DECLS
