@@ -39,6 +39,17 @@
 #include "core/display-private.h"
 #include "meta/util.h"
 
+enum
+{
+  PROP_0,
+
+  PROP_BACKEND,
+
+  N_PROPS
+};
+
+static GParamSpec *props[N_PROPS] = { 0 };
+
 static GQuark quark_tool_settings = 0;
 
 typedef struct _MetaInputSettingsPrivate MetaInputSettingsPrivate;
@@ -66,6 +77,8 @@ struct _DeviceMappingInfo
 
 struct _MetaInputSettingsPrivate
 {
+  MetaBackend *backend;
+
   ClutterSeat *seat;
   gulong monitors_changed_id;
 
@@ -375,6 +388,10 @@ do_update_pointer_accel_profile (MetaInputSettings          *input_settings,
     input_settings_class->set_mouse_accel_profile (input_settings,
                                                    device,
                                                    profile);
+  else if (settings == priv->touchpad_settings)
+    input_settings_class->set_touchpad_accel_profile (input_settings,
+                                                      device,
+                                                      profile);
   else if (settings == priv->trackball_settings)
     input_settings_class->set_trackball_accel_profile (input_settings,
                                                        device,
@@ -930,12 +947,18 @@ update_trackball_scroll_button (MetaInputSettings  *input_settings,
   MetaInputSettingsPrivate *priv;
   guint button;
   gboolean button_lock;
+  ClutterInputCapabilities caps;
 
   priv = meta_input_settings_get_instance_private (input_settings);
   input_settings_class = META_INPUT_SETTINGS_GET_CLASS (input_settings);
 
-  if (device && !input_settings_class->is_trackball_device (input_settings, device))
-    return;
+  if (device)
+    {
+      caps = clutter_input_device_get_capabilities (device);
+
+      if ((caps & CLUTTER_INPUT_CAPABILITY_TRACKBALL) == 0)
+        return;
+    }
 
   /* This key is 'i' in the schema but it also specifies a minimum
    * range of 0 so the cast here is safe. */
@@ -953,8 +976,9 @@ update_trackball_scroll_button (MetaInputSettings  *input_settings,
       for (l = priv->devices; l; l = l->next)
         {
           device = l->data;
+          caps = clutter_input_device_get_capabilities (device);
 
-          if (input_settings_class->is_trackball_device (input_settings, device))
+          if ((caps & CLUTTER_INPUT_CAPABILITY_TRACKBALL) != 0)
             input_settings_class->set_scroll_button (input_settings, device, button, button_lock);
         }
     }
@@ -1002,19 +1026,6 @@ update_tablet_keep_aspect (MetaInputSettings  *input_settings,
   if (!info)
     return;
 
-#ifdef HAVE_LIBWACOM
-  {
-    WacomDevice *wacom_device;
-
-    wacom_device = meta_input_device_get_wacom_device (META_INPUT_DEVICE (device));
-
-    /* Keep aspect only makes sense in external tablets */
-    if (wacom_device &&
-        libwacom_get_integration_flags (wacom_device) != WACOM_DEVICE_INTEGRATED_NONE)
-      return;
-  }
-#endif
-
   keep_aspect = g_settings_get_boolean (settings, "keep-aspect");
 
   if (keep_aspect)
@@ -1038,19 +1049,6 @@ update_tablet_mapping (MetaInputSettings  *input_settings,
        CLUTTER_INPUT_CAPABILITY_TABLET_TOOL) == 0)
     return;
 
-#ifdef HAVE_LIBWACOM
-  {
-    WacomDevice *wacom_device;
-
-    wacom_device = meta_input_device_get_wacom_device (META_INPUT_DEVICE (device));
-
-    /* Tablet mapping only makes sense on external tablets */
-    if (wacom_device &&
-        (libwacom_get_integration_flags (wacom_device) != WACOM_DEVICE_INTEGRATED_NONE))
-      return;
-  }
-#endif
-
   input_settings_class = META_INPUT_SETTINGS_GET_CLASS (input_settings);
   mapping = g_settings_get_enum (settings, "mapping");
 
@@ -1164,6 +1162,8 @@ meta_input_settings_changed_cb (GSettings  *settings,
         update_device_speed (input_settings, NULL);
       else if (strcmp (key, "natural-scroll") == 0)
         update_device_natural_scroll (input_settings, NULL);
+      else if (strcmp (key, "accel-profile") == 0)
+        update_pointer_accel_profile (input_settings, settings, NULL);
       else if (strcmp (key, "tap-to-click") == 0)
         update_touchpad_tap_enabled (input_settings, NULL);
       else if (strcmp (key, "tap-button-map") == 0)
@@ -1704,6 +1704,26 @@ meta_input_settings_constructed (GObject *object)
   load_keyboard_a11y_settings (input_settings);
 }
 
+static void
+meta_input_settings_set_property (GObject      *object,
+                                  guint         prop_id,
+                                  const GValue *value,
+                                  GParamSpec   *pspec)
+{
+  MetaInputSettings *settings = META_INPUT_SETTINGS (object);
+  MetaInputSettingsPrivate *priv =
+    meta_input_settings_get_instance_private (settings);
+
+  switch (prop_id)
+    {
+    case PROP_BACKEND:
+      priv->backend = g_value_get_object (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
 static void
 meta_input_settings_class_init (MetaInputSettingsClass *klass)
 {
@@ -1711,6 +1731,17 @@ meta_input_settings_class_init (MetaInputSettingsClass *klass)
 
   object_class->dispose = meta_input_settings_dispose;
   object_class->constructed = meta_input_settings_constructed;
+  object_class->set_property = meta_input_settings_set_property;
+
+  props[PROP_BACKEND] =
+    g_param_spec_object ("backend",
+                         "backend",
+                         "MetaBackend",
+                         META_TYPE_BACKEND,
+                         G_PARAM_WRITABLE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS);
+  g_object_class_install_properties (object_class, N_PROPS, props);
 
   quark_tool_settings =
     g_quark_from_static_string ("meta-input-settings-tool-settings");
@@ -1857,3 +1888,12 @@ meta_input_settings_get_kbd_a11y_settings (MetaInputSettings   *input_settings,
 
   *a11y_settings = priv->kbd_a11y_settings;
 }
+
+MetaBackend *
+meta_input_settings_get_backend (MetaInputSettings *settings)
+{
+  MetaInputSettingsPrivate *priv =
+    meta_input_settings_get_instance_private (settings);
+
+  return priv->backend;
+}
