@@ -578,6 +578,9 @@ meta_kms_device_finalize (GObject *object)
   MetaKmsDevice *device = META_KMS_DEVICE (object);
 
   g_free (device->path);
+  g_free (device->driver_name);
+  g_free (device->driver_description);
+  g_list_free (device->fallback_modes);
   g_list_free (device->crtcs);
   g_list_free (device->connectors);
   g_list_free (device->planes);
