@@ -26,7 +26,6 @@
 
 #include "core/place.h"
 
-#include <gdk/gdk.h>
 #include <math.h>
 #include <stdlib.h>
 
@@ -84,7 +83,9 @@ find_next_cascade (MetaWindow *window,
                    int        *new_x,
                    int        *new_y)
 {
-  MetaBackend *backend = meta_get_backend ();
+  MetaDisplay *display = meta_window_get_display (window);
+  MetaContext *context = meta_display_get_context (display);
+  MetaBackend *backend = meta_context_get_backend (context);
   GList *tmp;
   GList *sorted;
   int cascade_x, cascade_y;
@@ -665,7 +666,9 @@ meta_window_place (MetaWindow        *window,
                    int               *new_x,
                    int               *new_y)
 {
-  MetaBackend *backend = meta_get_backend ();
+  MetaDisplay *display = meta_window_get_display (window);
+  MetaContext *context = meta_display_get_context (display);
+  MetaBackend *backend = meta_context_get_backend (context);
   GList *windows = NULL;
   MetaLogicalMonitor *logical_monitor;
 
@@ -825,8 +828,7 @@ meta_window_place (MetaWindow        *window,
     }
 
   /* Find windows that matter (not minimized, on same workspace
-   * as placed window, may be shaded - if shaded we pretend it isn't
-   * for placement purposes)
+   * as placed window)
    */
   {
     GSList *all_windows;
