@@ -28,9 +28,10 @@ clutter_perf_fps_init (void)
   g_random_set_seed (12345678);
 }
 
-static void perf_stage_after_paint_cb (ClutterStage        *stage,
-                                       ClutterPaintContext *paint_context,
-                                       gpointer            *data);
+static void perf_stage_after_paint_cb (ClutterStage     *stage,
+                                       ClutterStageView *view,
+                                       ClutterFrame     *frame,
+                                       gpointer         *data);
 static gboolean perf_fake_mouse_cb (gpointer stage);
 
 static inline void
@@ -53,9 +54,10 @@ clutter_perf_fps_report (const gchar *id)
 }
 
 static void
-perf_stage_after_paint_cb (ClutterStage        *stage,
-                           ClutterPaintContext *paint_context,
-                           gpointer            *data)
+perf_stage_after_paint_cb (ClutterStage     *stage,
+                           ClutterStageView *view,
+                           ClutterFrame     *frame,
+                           gpointer         *data)
 {
   if (!testtimer)
     testtimer = g_timer_new ();
@@ -104,7 +106,6 @@ static gboolean perf_fake_mouse_cb (gpointer stage)
       device = clutter_seat_get_pointer (seat);
 
       event2->crossing.stage = stage;
-      event2->crossing.source = stage;
       event2->crossing.x = 10;
       event2->crossing.y = 10;
       event2->crossing.related = NULL;
