@@ -71,6 +71,9 @@ item(_MUTTER_TIMESTAMP_PING)
 item(_MUTTER_FOCUS_SET)
 item(_MUTTER_SENTINEL)
 item(_MUTTER_VERSION)
+item(_MUTTER_FRAME_FOR)
+item(_MUTTER_FRAME_EXTENTS)
+item(_MUTTER_NEEDS_FRAME)
 item(WM_CLIENT_MACHINE)
 item(MANAGER)
 item(TARGETS)
@@ -107,7 +110,6 @@ item(_NET_SUPPORTED)
 item(_NET_WM_NAME)
 item(_NET_CLOSE_WINDOW)
 item(_NET_WM_STATE)
-item(_NET_WM_STATE_SHADED)
 item(_NET_WM_STATE_MAXIMIZED_HORZ)
 item(_NET_WM_STATE_MAXIMIZED_VERT)
 item(_NET_WM_DESKTOP)
