@@ -1528,32 +1528,22 @@ handle_other_xevent (MetaX11Display *x11_display,
 
           window = meta_window_x11_new (display, event->xmaprequest.window,
                                         FALSE, META_COMP_EFFECT_CREATE);
-          /* The window might have initial iconic state, but this is a
-           * MapRequest, fall through to ensure it is unminimized in
-           * that case.
-           */
         }
-      else if (frame_was_receiver)
+      else
         {
-          break;
-        }
+          meta_verbose ("MapRequest on %s mapped = %d minimized = %d",
+                        window->desc, window->mapped, window->minimized);
 
-      /* Double check that creating the MetaWindow succeeded */
-      if (window == NULL)
-        break;
-
-      meta_verbose ("MapRequest on %s mapped = %d minimized = %d",
-                    window->desc, window->mapped, window->minimized);
-
-      if (window->minimized)
-        {
-          meta_window_unminimize (window);
-          if (window->workspace != workspace_manager->active_workspace)
+          if (window->minimized && !frame_was_receiver)
             {
-              meta_verbose ("Changing workspace due to MapRequest mapped = %d minimized = %d",
-                            window->mapped, window->minimized);
-              meta_window_change_workspace (window,
-                                            workspace_manager->active_workspace);
+              meta_window_unminimize (window);
+              if (window->workspace != workspace_manager->active_workspace)
+                {
+                  meta_verbose ("Changing workspace due to MapRequest mapped = %d minimized = %d",
+                                window->mapped, window->minimized);
+                  meta_window_change_workspace (window,
+                                                workspace_manager->active_workspace);
+                }
             }
         }
       break;
